{*******************************************************************************}
{  TXCheckList v1.00                                                            }
{  Programmed by Vegard A. Larsen (vegard@xaltra.net)                           }
{  Copyright  2001 Vegard A. Larsen. All rights reserved.                      }
{  URL: http://www.xaltra.net                                                   }
{*******************************************************************************}
{ Explanation of use, license and other information: See readme.htm file        }

unit XCheckList;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TXCheckList = class(TGraphicControl)
  private
    FItems: TStringList;
    FTrans: Boolean;
    FCheckedImage: TBitmap;
    FActiveImage: TBitmap;
    FMargin: Integer;
    FChecked: Integer;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
  protected
    procedure SetItems(Value: TStringList);
    function GetItems: TStringList;
    procedure SetTrans(Value: Boolean);
    procedure SetMargin(Value: Integer);
    procedure SetChecked(Value: Integer);
    procedure SetCheckedImage(Value: TBitmap);
    procedure SetActiveImage(Value: TBitmap);
    function GetCheckedImage: TBitmap;
    function GetActiveImage: TBitmap;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
    property Canvas;
    property ClientRect;
  published
    property Items: TStringList read GetItems write SetItems;
    property Transparent: Boolean read FTrans write SetTrans;
    property CheckedImage: TBitmap read FCheckedImage write SetCheckedImage;
    property ActiveImage: TBitmap read FActiveImage write SetActiveImage;
    property Margin: Integer read FMargin write SetMargin;
    property Checked: Integer read FChecked write SetChecked;
    property Color;
    property Font;
    property Width;
    property Height;
    property Left;
    property Top;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Xaltra', [TXCheckList]);
end;

{$R XCheckList.res}

constructor TXCheckList.Create(AOwner: TComponent);
begin
  inherited;

  FActiveImage := TBitmap.Create;
  FActiveImage.Transparent := True;
  FCheckedImage := TBitmap.Create;
  FCheckedImage.Transparent := True;

  Checked := 0;
  Margin := 4;

  Width := 185;
  Height := 89;
end;

destructor TXCheckList.Destroy;
begin
  FItems.Destroy;

  if FActiveImage <> nil then
    FActiveImage.Destroy;
  if FCheckedImage <> nil then
    FCheckedImage.Destroy;

  inherited;
end;

procedure TXCheckList.SetItems(Value: TStringList);
begin
  if FItems = nil then
    FItems := TStringList.Create;
  FItems.Assign(Value);
  Invalidate;
end;

function TXCheckList.GetItems: TStringList;
begin
  if FItems = nil then
    FItems := TStringList.Create;
  Result := FItems
end;

procedure TXCheckList.SetTrans(Value: Boolean);
begin
  FTrans := Value;
  Invalidate;
end;

procedure TXCheckList.SetMargin(Value: Integer);
begin
  FMargin := Value;
  Invalidate;
end;

procedure TXCheckList.SetChecked(Value: Integer);
begin
  FChecked := Value;
  Invalidate;
end;

procedure TXCheckList.SetCheckedImage(Value: TBitmap);
begin
  if FCheckedImage = nil then
    FCheckedImage := TBitmap.Create;
  FCheckedImage.Assign(Value);
  FCheckedImage.Transparent := True;
  Invalidate;
end;

procedure TXCheckList.SetActiveImage(Value: TBitmap);
begin
  if FActiveImage = nil then
    FActiveImage := TBitmap.Create;
  FActiveImage.Assign(Value);
  FActiveImage.Transparent := True;
  Invalidate;
end;

function TXCheckList.GetCheckedImage: TBitmap;
begin
  if FCheckedImage = nil then
    FCheckedImage := TBitmap.Create;
  FCheckedImage.Transparent := True;
  Result := FCheckedImage;
end;

function TXCheckList.GetActiveImage: TBitmap;
begin
  if FActiveImage = nil then
    FActiveImage := TBitmap.Create;
  FActiveImage.Transparent := True;
  Result := FActiveImage;
end;

procedure TXCheckList.Paint;

  function GetItemHeight(Rect: TRect; Items: Integer): Byte;
  var
    h: Integer;
  begin
    h := Rect.Bottom - Rect.Top;
    if (Items <> 0) and (h <> 0) then
      Result := h div Items
    else
      Result := 0;
  end;

  procedure PaintItem(Canvas: TCanvas; Item, h, MaxW: Integer);
  var Top: Integer;
  begin
    if Item < Checked then
    begin
      if CheckedImage <> nil then
      begin
        Top := (Item * h) + (h div 2) - (CheckedImage.Height div 2);
        Canvas.Draw(0, Top, CheckedImage);
      end;
    end
    else
    if Item = Checked then
      if ActiveImage <> nil then
      begin
        Top := (Item * h) + (h div 2) - (ActiveImage.Height div 2);
        Canvas.Draw(0, Top, ActiveImage);
      end;
    Top := (Item * h) + (h div 2) - (Canvas.TextHeight(Items[Item]) div 2);
    Canvas.TextOut(MaxW + Margin, Top, Items[Item]);
  end;

var
  Rect: TRect;
  h, i, MaxImageW: Integer;
begin
  inherited;
  Canvas.Lock;
  Rect := ClientRect;
  h := GetItemHeight(Rect, Items.Count);

  MaxImageW := 0;

  if (ActiveImage <> nil) and (CheckedImage <> nil) then
    if ActiveImage.Width > CheckedImage.Width then
      MaxImageW := ActiveImage.Width
    else
      MaxImageW := CheckedImage.Width;

  Canvas.Brush.Color := Color;
  if not Transparent then
    Canvas.FillRect(Rect);
  Canvas.Font := Font;
  Canvas.Brush.Style := bsClear;

  if h <> 0 then
    for i := 0 to Items.Count - 1 do
      PaintItem(Canvas, i, h, MaxImageW);

  Canvas.Unlock;
end;

procedure TXCheckList.CMFontChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

end.
